package org.gidea.scada.monitor.udp.config;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.gidea.scada.monitor.config.RabbitMQConfig;
import org.gidea.scada.monitor.udp.entity.CjdMessage;
import org.gidea.scada.monitor.udp.utils.CrcUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.integration.annotation.Filter;
import org.springframework.integration.annotation.Router;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.annotation.Transformer;
import org.springframework.integration.dsl.IntegrationFlow;
import org.springframework.integration.dsl.IntegrationFlows;
import org.springframework.integration.ip.dsl.Udp;
import org.springframework.integration.ip.udp.UnicastSendingMessageHandler;
import org.springframework.integration.support.MessageBuilder;
import org.springframework.messaging.Message;
import org.springframework.messaging.handler.annotation.Headers;
import org.springframework.messaging.handler.annotation.Payload;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.*;
import java.util.LinkedHashMap;
import java.util.Map;

import static org.gidea.scada.monitor.utils.RabbitExchangeEnum.*;

@Slf4j
@Configuration
public class UdpServerConfig {
    @Value("${upd.server.port:23005}")
    private Integer listeningPort;
    @Resource
    private RabbitMQConfig rabbitMQConfig;

    private Map<String, String> ipMap = new LinkedHashMap<>();
    private String cjdFile = System.getProperty("user.dir") + "/config/cjd.json";

    @PostConstruct
    private void init() {
        if (new File(cjdFile).exists()) {
            String line;
            try {
                // Read the JSON file
                BufferedReader reader = new BufferedReader(new FileReader(cjdFile));
                StringBuilder jsonData = new StringBuilder();
                while (true) {
                    if (!((line = reader.readLine()) != null)) {
                        break;
                    }
                    jsonData.append(line);
                }
                reader.close();
                Map<String, String> data = JSONObject.parseObject(jsonData.toString(), new TypeReference<Map<String, String>>() {
                });
                ipMap.putAll(data);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

    /**
     * UDP消息接收服务
     */
    @Bean
    public IntegrationFlow integrationFlow() {
        log.info("UDP服务启动成功，端口号为: {}", listeningPort);
        return IntegrationFlows.from(Udp.inboundAdapter(listeningPort)).channel("udpChannel").get();
    }

    /**
     * 转换器
     */
    @Transformer(inputChannel = "udpChannel", outputChannel = "udpFilter")
    public CjdMessage transformer(@Payload byte[] payload) {
        String message = new String(payload);
        // todo 进行数据转换
        try {
            if (message.endsWith("\n")) {
                message = message.substring(0, message.length() - 1);
            }
            String tempCrc = message.substring(0, 4);
            String tempMessage = message.substring(5);
            String crc = CrcUtils.getCrc(tempMessage.getBytes());
            // 数据校验
            if (crc.equalsIgnoreCase(tempCrc)) {
                CjdMessage cjdMessage = JSONObject.parseObject(tempMessage, CjdMessage.class);
                return cjdMessage;
            }
        } catch (Exception e) {
            log.error("解析消息失败！", e);
        }
        log.info("无法解析消息：{}", message);
        return null;
    }


    /**
     * 过滤器
     */
    @Filter(inputChannel = "udpFilter", outputChannel = "udpRouter")
    public boolean filter(CjdMessage message) {
        if (message == null) {
            return false;
        }
        return true;
    }

    /**
     * 路由分发处理器:可以进行分发消息被那个处理器进行处理
     */
    @Router(inputChannel = "udpRouter")
    public String router(CjdMessage message) {
        // todo 筛选，走那个处理器
        switch (message.getCode()) {
            case 0:
                return "messageHandle0";
            case 1:
                return "messageHandle1";
        }
        return "messageHandle";
    }

    /**
     * 消息处理
     */
    @ServiceActivator(inputChannel = "messageHandle")
    private void messageHandle(CjdMessage message) {
        log.info("统一消息处理：{}", JSONObject.toJSONString(message));
    }

    /**
     * 消息处理
     */
    @ServiceActivator(inputChannel = "messageHandle0", outputChannel = "responseHandle")
    private CjdMessage messageHandle0(CjdMessage message) {
        log.info("处理采集端主动推送数据：{}", JSONObject.toJSONString(message));
        CjdMessage cjdMessage = new CjdMessage(message);
        cjdMessage.setCode(1);
        if (10000 == message.getCmd()) {
            ipMap.put(message.getBscode(), message.getIp());

            File file = new File(cjdFile);
            if (!file.exists()) {
                try {
                    if (!file.getParentFile().exists()) {
                        file.getParentFile().mkdirs();
                    }
                    file.createNewFile();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
            String string = JSONObject.toJSONString(ipMap, true);
            // 将 JSON 字符串写入文件
            try (FileWriter fileWriter = new FileWriter(file)) {
                fileWriter.write(string);
                fileWriter.flush();
                fileWriter.close();
                System.out.println("JSON 数据已成功写入文件。");
            } catch (IOException e) {
                e.printStackTrace();
            }

            JSONObject data = rabbitMQConfig.getRabbitInfo();
            data.put("prefix", MonitorDeviceData.exchangeName);
            data.put("serviceExchange", MonitorCjdService.exchangeName);
            data.put("clientExchange", MonitorCjdClient.exchangeName);
            cjdMessage.put("mqtt", data);
        }
        return cjdMessage;
    }

    /**
     * 消息处理
     */
    @ServiceActivator(inputChannel = "messageHandle1")
    private CjdMessage messageHandle1(CjdMessage message) {
        log.info("处理采集端响应结果数据：{}", JSONObject.toJSONString(message));
        return null;
    }

    /**
     * 消息响应发送
     */
    @ServiceActivator(inputChannel = "responseHandle")
    private void responseHandle(String message, @Headers Map<String, Object> headers) {
        if (message == null || "".equals(message)) {
            return;
        }
        // 获取来源IP，可以进行IP过滤
        String ipAddress = headers.get("ip_address").toString();
        // 获取来源Port
        Integer port = Integer.parseInt(headers.get("ip_port").toString());
        // 创建消息
        Message<String> udpMessage = MessageBuilder.withPayload(message).setHeader("ip_address", ipAddress).setHeader("ip_port", port).build();
        // 创建发送器
        UnicastSendingMessageHandler messageHandler = new UnicastSendingMessageHandler(ipAddress, port);
        // 发送消息
        messageHandler.handleMessage(udpMessage);
    }

    public String getCjdIp(String cjdId) {
        return ipMap.get(cjdId);
    }
}

